/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:
 known Problems: none
 Version:        24.10.2007
 Description:    Timer Routinen

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#include "config.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include "usart.h"
#include "stack.h"
#include "cmd.h"
#include "timer.h"
#include "analog.h"

volatile unsigned long time;
volatile unsigned long time_watchdog = 0;
volatile unsigned int counter;

//----------------------------------------------------------------------------
//Diese Routine startet und initialisiert den Timer
void timer_init (void)
{
	#if EXTCLOCK==1				//Wenn externer Quarz als Takt -> welche Frequenz?
		#if defined (__AVR_ATmega644P__)
			//Asynchroner Modus ein, Oszillator an TOSC1 und TOSC2 aktiv
			ASSR |= (1<<AS2);
			TCCR2B = 0x05;
			while(ASSR & 0x11);
			//Capture/Compare-Interrupt aktiv
			TIMSK2 |= (1<<OCIE2A);
		#else
			//Asynchroner Modus ein, Oszillator an TOSC1 und TOSC2 aktiv
			ASSR  = (1<<AS2);
			//CTC-Modus an (Clear Timer on Compare Match)
			TCCR2 = (1<<WGM21);			
			//dieser Wert ergibt eine Sekunde Periodendauer
			OCR2  = 31;	
			//lsche Prescaler 2				
			SFIE  = (1<<PSR2);			
			//Starte Timer 2 mit Prescaler gleich 1/1024
			TCCR2 |= (1<<CS22)|(1<<CS21)|(1<<CS20); 
			while(ASSR & 0x07);
			//Capture/Compare-Interrupt aktiv
			TIMSK = (1<<OCIE2);
		#endif
	#else				//if EXTCLOCK==0 --> timer1
		#if defined (__AVR_ATmega644P__)
			
			TCCR1B |= (1<<WGM12)|(1<<CS10 | 0<<CS11 | 1<<CS12);	//CTC(compare match mode)|clk/1024 prescaler
			TCNT1 = 0;		//Zhlerregister = 0
			OCR1A = (F_CPU / 1024) - 1; //Zhlgrenze fr UEberlauf
			TIMSK1 |= (1 << OCIE1A);	//ISR wenn Grenze A (OCR1A) erreicht
		#else
			TCCR1B |= (1<<WGM12) | (1<<CS10 | 0<<CS11 | 1<<CS12);	//CTC(compare match mode)|clk/1024 prescaler
			TCNT1 = 0;		//Zhlerregister = 0
			OCR1A = (F_CPU / 1024) - 1; //Zhlgrenze fr UEberlauf
			TIMSK |= (1 << OCIE1A);	//ISR wenn Grenze A (OCR1A) erreicht
		#endif	//Atmega
	#endif		//EXTCLOCK
	//Beginn: Mein Sensorclock!
	#if defined (__AVR_ATmega644P__)
		TCCR0A       |= (1<<WGM01);	// Timer im CTC-Mode, Top=OCR0; 8 bit timer
		TCCR0A        &= ~(1<<WGM00);	// kein PWM
		TCCR0B        |= (1<<CS00);
		TCCR0B        |= (1<<CS01);
		TCCR0B        &= ~(1<<CS02);		// Prescaler 64 -->16MHz/64 = 250 kHz
		OCR0A          = 25;			// 250 kHz/25 = 10 kHz Schalttakt, aber 1xAn+1xAus= Periode  
										// 10 kHz/2 = 5 kHz = Fclock
										//jedes Mal wenn der Timer 239 erreicht gibt es einen Interrupt
		TIMSK0        |= (1<<OCIE0A);
	#else
		TCCR0        |= (1<<WGM01);	// Timer im CTC-Mode, Top=OCR0; 8 bit timer
		TCCR0        &= ~(1<<WGM00);	// kein PWM
		TCCR0        |= (1<<CS00);
		TCCR0        |= (1<<CS01);
		TCCR0        &= ~(1<<CS02);		// Prescaler 64 -->16MHz/64 = 250 kHz
		OCR0          = 25;			// 250 kHz/25 = 10 kHz Schalttakt, aber 1xAn+1xAus= Periode  
									// 10 kHz/2 = 5 kHz = Fclock
									//jedes Mal wenn der Timer 239 erreicht gibt es einen Interrupt
		TIMSK        |= (1<<OCIE0);	// Tmr/Cntr0 CompareA interrupt enabled
	#endif
	
/*	
Sensor braucht mindestens 5 kHz als CLK-Takt
z.B.: Prescaler 1/64; OCR0 = 25;

16 MHz/64 = 250 kHz --> 250 kHz/25 = 10 kHz --> 10 kHz/2 = 5kHz

oder
Prescaler 1/8; OCR0 = 200;
16MHz/8 = 2 MHz --> 2 MHz/200 = 10 kHz --> 10 kHz/2 = 5 kHz
*/		
//Ende: Mein Sensorclock

return;
};

//----------------------------------------------------------------------------
//Timer Interrupt
#if EXTCLOCK==1
	#if defined (__AVR_ATmega644P__)
    ISR (TIMER2_COMPA_vect)
	#else
    ISR (TIMER2_COMP_vect)
	#endif
#else
	
	#if defined (__AVR_ATmega644P__)
    ISR (PCINT3_vect)
	#else
    ISR (TIMER1_COMPA_vect)
	#endif
#endif
{
	//tick 1 second
	time++;
    if((stack_watchdog++) > WTT)  //emergency reset of the stack
    {
        RESET();
	}
    eth.timer = 1;
}

// ===  Nicht unterbrechbare ISR fr timer0
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

#if defined (__AVR_ATmega644P__)
    ISR (TIMER2_COMPB_vect)
	#else
    ISR (TIMER0_COMP_vect)
	#endif
 {		// Vektor 7, Prog.Addr. 0x0006
	PORTC ^=  (1<<1);			//  .. PC1 toggeln --> CLK
	
	
	
	//ADC Test___Anfang____________________
	#if Sensor
	if(counter<400){
	ADCSRA |= (1<<ADSC);
	sensorarray[counter]=ADC;
	}
	#endif	//Sensor
	//ADC Test___Ende____________________
	counter++;
	
	if(counter%898 == 0){		//%x --> jede 898/2te = 449. Periode
		PORTC  |= (1<<2);		//PC2 an --> SI
		counter = 0;
	}
	if(counter%898 == 1){		//%x --> jede 449. Periode
		PORTC  &= ~(1<<2);		//PC2 aus --> SI
		
	}
    return;
 }       // Ende ISR(TIM0_COMPB_vect)
